@@ -34,7 +34,8 @@ function createComposerWindow(myComposer) {
   ];
   if (GraphiteConfig.showMyGraphs) {
     var saveButton = createToolbarButton('Save to My Graphs', 'save.gif', saveMyGraph);
-    topToolbar.splice(0, 0, saveButton);
+    var deleteButton = createToolbarButton('Delete from My Graphs', 'delete.gif', deleteMyGraph);
+    topToolbar.splice(0, 0, saveButton, deleteButton);
   }
 
   var bottomToolbar = [
@@ -137,7 +138,7 @@ function createToolbarButton(tip, icon, handler) {
     style: "padding-left:10pt; background:transparent url(../content/img/" + icon + ") no-repeat scroll 0% 50%",
     handler: handler,
     handleMouseEvents: false,
-    text: "&nbsp; &nbsp; &nbsp; &nbsp;",
+    text: "&nbsp; &nbsp;",
     listeners: {
       render: function (button) {
         button.el.toolTip = new Ext.ToolTip({
@@ -368,6 +369,46 @@ function handleSaveMyGraphResponse(options, success, response) {
   });
 }
 
+
+function deleteMyGraph() {
+
+  Ext.MessageBox.prompt(
+    "Delete a saved My Graph", //title
+    "Please enter the name of the My Graph you wish to delete", //prompt message
+    function (button, text) { //handler
+      if (button != 'ok') {
+        return;
+      }
+
+      if (!text) {
+        Ext.Msg.alert("Invalid My Graph name!");
+	return;
+      }
+
+      Browser.trees.mygraphs.reload();
+      //Send the request
+      Ext.Ajax.request({
+        method: 'GET',
+        url: '../composer/mygraph/',
+        params: {action: 'delete', graphName: text},
+        callback: function (options, success, response) {
+          var message = success ? "Graph deleted successfully" : "There was an error performing the operation.";
+
+          Ext.Msg.show({
+            title: 'Delete My Graph',
+            msg: message,
+            buttons: Ext.Msg.OK
+          });
+        }
+      });
+    },
+    this,   //scope
+    false,  //multiline
+    Composer.state.myGraphName ? Composer.state.myGraphName : "" //default value
+  );
+
+}
+
 /* Graph Data dialog  */
 var GraphDataWindow = {
   create: function () {
@@ -442,7 +483,8 @@ var GraphDataWindow = {
                 menu: [
                   {text: 'Sum', handler: this.applyFuncToAll('sumSeries')},
                   {text: 'Difference', handler: this.applyFuncToAll('diffSeries')},
-                  {text: 'Average', handler: this.applyFuncToAll('averageSeries')}
+                  {text: 'Average', handler: this.applyFuncToAll('averageSeries')},
+                  {text: 'Sum using wildcards', handler: this.applyFuncToEachWithInput('sumSeriesWithWildcards', 'Please enter a comma separated list of numbers specifying the locations in the name to place wildcards')}
                 ]
               }, {
                 text: 'Transform',
@@ -451,7 +493,9 @@ var GraphDataWindow = {
                   {text: 'Offset', handler: this.applyFuncToEachWithInput('offset', 'Please enter the value to offset Y-values by')},
                   {text: 'Derivative', handler: this.applyFuncToEach('derivative')},
                   {text: 'Integral', handler: this.applyFuncToEach('integral')},
-                  {text: 'Non-negative Derivative', handler: this.applyFuncToEach('nonNegativeDerivative')}
+                  {text: 'Non-negative Derivative', handler: this.applyFuncToEachWithInput('nonNegativeDerivative', "Please enter a maximum value if this metric is a wrapping counter (or just leave this blank)",true)},
+                  {text: 'Log', handler: this.applyFuncToEachWithInput('log', 'Please enter a base')},
+                  {text: 'timeShift', handler: this.applyFuncToEachWithInput('timeShift', 'Shift this metric ___ back in time')}
                 ]
               }, {
                 text: 'Calculate',
@@ -471,7 +515,12 @@ var GraphDataWindow = {
                   {text: 'Highest Average Value', handler: this.applyFuncToEachWithInput('highestAverage', 'Draw the ___ metrics with the highest average value')},
                   {text: 'Lowest Average Value', handler: this.applyFuncToEachWithInput('lowestAverage', 'Draw the ___ metrics with the lowest average value')},
                   {text: 'Average Value Above', handler: this.applyFuncToEachWithInput('averageAbove', 'Draw all metrics whose average value is above ___')},
-                  {text: 'Average Value Above', handler: this.applyFuncToEachWithInput('averageBelow', 'Draw all metrics whose average value is below ___')}
+                  {text: 'Average Value Below', handler: this.applyFuncToEachWithInput('averageBelow', 'Draw all metrics whose average value is below ___')},
+                  {text: 'Maximum Value Above', handler: this.applyFuncToEachWithInput('maximumAbove', 'Draw all metrics whose maximum value is above ___')},
+                  {text: 'Maximum Value Below', handler: this.applyFuncToEachWithInput('maximumBelow', 'Draw all metrics whose maximum value is below ___')},
+                  {text: 'sortByMaxima', handler: this.applyFuncToEach('sortByMaxima')},
+                  {text: 'sortByMinima', handler: this.applyFuncToEach('sortByMinima')},
+                  {text: 'limit', handler: this.applyFuncToEachWithInput('limit', 'Limit to first ___ of a list of metrics')}
                 ]
               }, {
                 text: 'Special',
@@ -481,7 +530,8 @@ var GraphDataWindow = {
                   {text: 'Draw non-zero As Infinite', handler: this.applyFuncToEach('drawAsInfinite')},
                   {text: 'Line Width', handler: this.applyFuncToEachWithInput('lineWidth', 'Please enter a line width for this graph target')},
                   {text: 'Dashed Line', handler: this.applyFuncToEach('dashed')},
-                  {text: 'Keep Last Value', handler: this.applyFuncToEach('keepLastValue')}
+                  {text: 'Keep Last Value', handler: this.applyFuncToEach('keepLastValue')},
+                  {text: 'Substring', handler: this.applyFuncToEachWithInput('substr', 'Enter a starting position')}
                 ]
               }
             ]
@@ -593,14 +643,14 @@ var GraphDataWindow = {
     return applyFunc;
   },
 
-  applyFuncToEachWithInput: function (funcName, question) {
+  applyFuncToEachWithInput: function (funcName, question, allowBlank) {
     function applyFunc() {
       Ext.MessageBox.prompt(
         "Input Required", //title
         question, //message
         function (button, inputValue) { //handler
-          if (button == 'ok' && inputValue != '') {
-            if (funcName == 'alias') { //SPECIAL CASE HACK
+          if (button == 'ok' && (allowBlank || inputValue != '')) {
+            if (funcName == 'alias' && inputValue != '') { //SPECIAL CASE HACK
               inputValue = '"' + inputValue + '"';
             }
             this.applyFuncToEach(funcName, inputValue)();
