@@ -18,6 +18,7 @@ from calendar import timegm
 from urllib import unquote_plus
 from ConfigParser import SafeConfigParser
 from django.conf import settings
+from graphite.render.datatypes import TimeSeries
 
 colorAliases = {
   'black' : (0,0,0),
@@ -74,11 +75,27 @@ xAxisConfigs = (
   dict(seconds=864000,minorGridUnit=YEAR, minorGridStep=1,  majorGridUnit=YEAR, majorGridStep=1,  labelUnit=YEAR, labelStep=1,  format="%y"),
 )
 
+UnitSystems = {
+  'binary': (
+    ('Pi', 1024.0**5),
+    ('Ti', 1024.0**4),
+    ('Gi', 1024.0**3),
+    ('Mi', 1024.0**2),
+    ('Ki', 1024.0   )),
+  'si': (
+    ('P', 1000.0**5),
+    ('T', 1000.0**4),
+    ('G', 1000.0**3),
+    ('M', 1000.0**2),
+    ('K', 1000.0   )),
+  'none' : [],
+}
+
 
 class Graph:
   customizable = ('width','height','margin','bgcolor','fgcolor', \
                  'fontName','fontSize','fontBold','fontItalic', \
-                 'colorList','template')
+                 'colorList','template','YAxis')
 
   def __init__(self,**params):
     self.params = params
@@ -89,7 +106,7 @@ class Graph:
     self.userTimeZoneOffset = tz_difference( params.get('tz','') )
 
     self.area = {
-      'xmin' : self.margin,
+      'xmin' : self.margin + 10, # Need extra room when the time is near the left edge
       'xmax' : self.width - self.margin,
       'ymin' : self.margin,
       'ymax' : self.height - self.margin,
@@ -221,7 +238,7 @@ class Graph:
   def drawLegend(self,elements): #elements is [ (name,color), (name,color), ... ]
     longestName = sorted([e[0] for e in elements],key=len)[-1]
     extents = self.getExtents(longestName)
-    padding = 4
+    padding = 5
     boxSize = extents['maxHeight'] - 1
     lineHeight = extents['maxHeight'] + 1
     labelWidth = extents['width'] + 2 * (boxSize + padding)
@@ -279,9 +296,12 @@ class LineGraph(Graph):
                  ('title','vtitle','lineMode','lineWidth','hideLegend', \
                   'hideAxes','minXStep','hideGrid','majorGridLineColor', \
                   'minorGridLineColor','thickness','min','max', \
-                  'graphOnly','yMin','yMax','yStep','areaMode','drawNullAsZero','tz')
+                  'graphOnly','yMin','yMax','yLimit','yStep','areaMode', \
+                  'areaAlpha','drawNullAsZero','tz', 'YAxis','pieMode', \
+                  'yUnitSystem')
   validLineModes = ('staircase','slope')
   validAreaModes = ('none','first','all','stacked')
+  validPieModes = ('maximum', 'minimum', 'average')
 
   def drawGraph(self,**params):
     #API compatibilty hacks first
@@ -289,6 +309,7 @@ class LineGraph(Graph):
       params['hideLegend'] = True
       params['hideGrid'] = True
       params['hideAxes'] = True
+      params['YAxis'] = 'left' 
       params['title'] = ''
       params['vtitle'] = ''
       params['margin'] = 0
@@ -304,14 +325,23 @@ class LineGraph(Graph):
       params['yMax'] = params['max']
     if 'lineWidth' not in params and 'thickness' in params:
       params['lineWidth'] = params['thickness']
+    if 'YAxis' not in params:
+      params['YAxis'] = 'left'
+    if 'yUnitSystem' not in params:
+      params['yUnitSystem'] = 'si'
     self.params = params
-
+    # When Y Axis is labeled on the right, we subtract x-axis positions from the max, 
+    # instead of adding to the minimum
+    if self.params.get('YAxis') == 'right': 
+      self.margin = self.width
     #Now to setup our LineGraph specific options
     self.lineWidth = float( params.get('lineWidth', 1.2) )
     self.lineMode = params.get('lineMode','slope').lower()
     assert self.lineMode in self.validLineModes, "Invalid line mode!"
     self.areaMode = params.get('areaMode','none').lower()
     assert self.areaMode in self.validAreaModes, "Invalid area mode!"
+    self.pieMode = params.get('pieMode', 'maximum').lower()
+    assert self.pieMode in self.validPieModes, "Invalid pie mode!"
 
     for series in self.data:
       series.color = self.colors.next()
@@ -325,7 +355,7 @@ class LineGraph(Graph):
       self.drawVTitle( str(params['vtitle']) )
     self.setFont()
 
-    if not params.get('hideLegend', len(self.data) > 10):
+    if not params.get('hideLegend', len(self.data) > settings.LEGEND_MAX_ITEMS):
       elements = [ (series.name,series.color) for series in self.data ]
       self.drawLegend(elements)
 
@@ -408,6 +438,16 @@ class LineGraph(Graph):
     self.ctx.clip()
     self.ctx.set_line_width(originalWidth)
 
+    if self.params.get('areaAlpha') and self.areaMode == 'first':
+      alphaSeries = TimeSeries(None, self.data[0].start, self.data[0].end, self.data[0].step, [x for x in self.data[0]])
+      alphaSeries.xStep = self.data[0].xStep
+      alphaSeries.color = self.data[0].color
+      try:
+        alphaSeries.options['alpha'] = float(self.params['areaAlpha'])
+      except ValueError:
+        pass
+      self.data.insert(0, alphaSeries)
+
     for series in self.data:
 
       if series.options.has_key('lineWidth'): # adjusts the lineWidth of this line if option is set on the series
@@ -420,7 +460,7 @@ class LineGraph(Graph):
 
       x = float(self.area['xmin']) + (self.lineWidth / 2.0)
       y = float(self.area['ymin'])
-      self.setColor( series.color )
+      self.setColor( series.color, series.options.get('alpha') or 1.0)
 
       fromNone = True
 
@@ -439,6 +479,7 @@ class LineGraph(Graph):
 
         else:
           y = self.area['ymax'] - ((float(value) - self.yBottom) * self.yScaleFactor)
+          if y < 0: y = 0 
 
           if series.options.has_key('drawAsInfinite') and value > 0:
             self.ctx.move_to(x, self.area['ymax'])
@@ -530,9 +571,12 @@ class LineGraph(Graph):
     if 'yMax' in self.params:
       yMaxValue = self.params['yMax']
 
-    if 'yMin' in self.params:
-      yMinvalue = self.params['yMin']
+    if 'yLimit' in self.params and self.params['yLimit'] < yMaxValue:
+      yMaxValue = self.params['yLimit']
 
+    if 'yMin' in self.params:
+      yMinValue = self.params['yMin']
+    
     yVariance = yMaxValue - yMinValue
 
     if math.floor(yVariance) == 0:
@@ -578,19 +622,36 @@ class LineGraph(Graph):
 
     if not self.params.get('hideAxes',False):
       #Create and measure the Y-labels
-      def makeLabel(yValue): #TODO beautify this!
-        if self.ySpan > 10:
-          return "%d " % int(yValue)
-        elif self.ySpan > 3:
-          return "%.1f " % float(yValue)
+
+      def makeLabel(yValue):
+        yValue, prefix = format_units(yValue,
+                system=self.params.get('yUnitSystem'))
+
+        ySpan, spanPrefix = format_units(self.ySpan, 
+                system=self.params.get('yUnitSystem'))
+
+        if ySpan > 10 or spanPrefix != prefix:
+          return "%d %s " % (int(yValue), prefix)
+
+        elif ySpan > 3:
+          return "%.1f %s " % (float(yValue), prefix)
+
         else:
-          return "%.2f " % float(yValue)
+          return "%.2f %s " % (float(yValue), prefix)
+
       self.yLabelValues = list( frange(self.yBottom,self.yTop,self.yStep) )
       self.yLabels = map(makeLabel,self.yLabelValues)
       self.yLabelWidth = max([self.getExtents(label)['width'] for label in self.yLabels])
-      xMin = self.margin + (self.yLabelWidth * 1.02)
-      if self.area['xmin'] < xMin:
-        self.area['xmin'] = xMin #scoot the graph over to the right just enough to fit the y-labels
+
+      if self.params.get('YAxis') == 'left': #scoot the graph over to the left just enough to fit the y-labels
+        xMin = self.margin + (self.yLabelWidth * 1.02)
+        if self.area['xmin'] < xMin:
+          self.area['xmin'] = xMin
+      else: #scoot the graph over to the right just enough to fit the y-labels
+        xMin = 0
+        xMax = self.margin - (self.yLabelWidth * 1.02)
+        if self.area['xmax'] >= xMax:
+          self.area['xmax'] = xMax
     else:
       self.yLabelValues = []
       self.yLabels = []
@@ -624,9 +685,15 @@ class LineGraph(Graph):
   def drawLabels(self):
     #Draw the Y-labels
     for value,label in zip(self.yLabelValues,self.yLabels):
-      x = self.area['xmin'] - (self.yLabelWidth * 0.02)
+      if self.params.get('YAxis') == 'left':
+        x = self.area['xmin'] - (self.yLabelWidth * 0.02)
+      else:
+        x = self.area['xmax'] + (self.yLabelWidth * 0.02) #Inverted for right side Y Axis
       y = self.area['ymax'] - ((value - self.yBottom) * self.yScaleFactor)
-      self.drawText(label,x,y,align='right',valign='middle')
+      if self.params.get('YAxis') == 'left':
+        self.drawText(label,x,y,align='right',valign='middle')
+      else:
+        self.drawText(label,x,y,align='left',valign='middle') #Inverted for right side Y Axis
     t = self.xLabelStep * math.ceil( float(self.startTime) / self.xLabelStep )
 
     #Draw the X-labels
@@ -664,18 +731,20 @@ class LineGraph(Graph):
     t = self.xMinorGridStep * math.ceil( float(self.startTime) / self.xMinorGridStep )
     while t < self.endTime:
       x = self.area['xmin'] + ((t - self.startTime) * self.xScaleFactor)
-      self.ctx.move_to(x, bottom)
-      self.ctx.line_to(x, top)
-      self.ctx.stroke()
+      if x < self.area['xmax']: 
+        self.ctx.move_to(x, bottom)
+        self.ctx.line_to(x, top)
+        self.ctx.stroke()
       t += self.xMinorGridStep
     self.ctx.set_line_width(0.33) #Now for the major grid lines
     self.setColor( self.params.get('majorGridLineColor',self.defaultMajorGridLineColor) )
     t = self.xMajorGridStep * math.ceil( float(self.startTime) / self.xMajorGridStep )
     while t < self.endTime:
       x = self.area['xmin'] + ((t - self.startTime) * self.xScaleFactor)
-      self.ctx.move_to(x, bottom)
-      self.ctx.line_to(x, top)
-      self.ctx.stroke()
+      if x < self.area['xmax']: 
+        self.ctx.move_to(x, bottom)
+        self.ctx.line_to(x, top)
+        self.ctx.stroke()
       t += self.xMajorGridStep
     #Draw side borders for our graph area
     self.ctx.set_line_width(0.5)
@@ -688,10 +757,11 @@ class LineGraph(Graph):
 
 class PieGraph(Graph):
   customizable = Graph.customizable + \
-                 ('title','valueLabels','valueLabelsMin','hideLegend')
+                 ('title','valueLabels','valueLabelsMin','hideLegend','pieLabels')
   validValueLabels = ('none','number','percent')
 
   def drawGraph(self,**params):
+    self.pieLabels = params.get('pieLabels', 'horizontal')
     self.total = sum( [t[1] for t in self.data] )
 
     self.slices = []
@@ -743,7 +813,7 @@ class PieGraph(Graph):
 
   def drawLabels(self):
     self.setFont()
-    self.setColor( self.foregroundColor )
+    self.setColor( 'black' )
     for slice in self.slices:
       if self.valueLabels == 'percent':
         if (slice['percent'] * 100.0) < self.valueLabelsMin: continue
@@ -758,9 +828,13 @@ class PieGraph(Graph):
       theta = slice['midAngle']
       x = self.x0 + (self.radius / 2.0 * math.cos(theta))
       y = self.y0 + (self.radius / 2.0 * math.sin(theta))
-      if theta > (math.pi / 2.0) and theta <= (3.0 * math.pi / 2.0):
-        theta -= math.pi
-      self.drawText( label, x, y, align='center', valign='middle', rotate=math.degrees(theta) )
+
+      if self.pieLabels == 'rotated':
+        if theta > (math.pi / 2.0) and theta <= (3.0 * math.pi / 2.0):
+          theta -= math.pi
+        self.drawText( label, x, y, align='center', valign='middle', rotate=math.degrees(theta) )
+      else:
+        self.drawText( label, x, y, align='center', valign='middle')
 
 
 GraphTypes = {
@@ -768,7 +842,8 @@ GraphTypes = {
   'pie' : PieGraph,
 }
 
-#Convience functions ... put these in milton?
+
+#Convience functions
 def closest(number,neighbors):
   distance = None
   closestNeighbor = None
@@ -823,3 +898,20 @@ def tz_difference(tz):
   except:
     os.environ['TZ'] = settings.TIME_ZONE
   return tz_delta
+
+def format_units(v, system="si"):
+  """Format the given value in standardized units.
+
+  ``system`` is either 'binary' or 'si'
+  
+  For more info, see:
+    http://en.wikipedia.org/wiki/SI_prefix
+    http://en.wikipedia.org/wiki/Binary_prefix
+  """
+
+  for prefix, size in UnitSystems[system]:
+    if abs(v) >= size:
+      v /= size
+      return v, prefix
+
+  return v, ""
